// Copyright 2015 Matthew Holt and The Caddy Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package envredisloader

import (
	"os"

	"github.com/caddyserver/caddy/v2"
	goredis "github.com/redis/go-redis/v9"
)

func init() {
	caddy.RegisterModule(ENVRedisLoader{})
}

// ENVRedisLoader can load Caddy configs over HTTP(S). It can adapt the config
// based on the Content-Type header of the HTTP response.
type ENVRedisLoader struct {
}

// CaddyModule returns the Caddy module information.
func (ENVRedisLoader) CaddyModule() caddy.ModuleInfo {
	return caddy.ModuleInfo{
		ID:  "caddy.config_loaders.env_redis",
		New: func() caddy.Module { return new(ENVRedisLoader) },
	}
}

func (ENVRedisLoader) LoadConfig(ctx caddy.Context) ([]byte, error) {
	ClientName := os.Getenv("NAME")
	redisOpts, err := goredis.ParseURL(os.Getenv("REDIS_URL"))
	if err != nil {
		return nil, err
	}
	rdb := goredis.NewClient(redisOpts)
	defer rdb.Close()
	val, err := rdb.Get(ctx, "caddy:config:"+ClientName).Bytes()
	if err != nil {
		return nil, err
	}
	return val, nil
}

var _ caddy.ConfigLoader = (*ENVRedisLoader)(nil)
